﻿-- Returns the ID of the zone the player is current in
function ZoneID()
	local currentZone = GetRealZoneText();
	local continentNames, key, val = { GetMapContinents() } ;
	for key, val in pairs(continentNames) do
		local continentZones = { GetMapZones(key) };
		for i,zone in ipairs(continentZones) do
			if zone == currentZone then
				return key
			end
		end
	end
	return nil
end

-- Used to find the Zero Difference used to calculate XP from low level mobs
function ZeroDifference(charLevel)
	for i,v in ipairs(ZD_Table) do
		if charLevel >= v[1] and charLevel <= v[2] then
			return v[3];
		end
	end
end

-- Calculates the XP gained from killing a mob
function MobXP(charLevel, mobLevel)
	if charLevel == nil then
		charLevel = UnitLevel("player");
	end
	if mobLevel == nil then 
		mobLevel = charLevel;
	end

	-- Find the highest level that yields no experience.
	local grayLevel = 0
	if charLevel < 6 then
		grayLevel = 0
	elseif charLevel < 40 then
		grayLevel = charLevel - floor(charLevel / 10) - 5
	elseif charLevel < 60 then
		grayLevel = charLevel - floor(charLevel / 5) - 1
	else
		grayLevel = charLevel - 9
	end
	
	if mobLevel <= grayLevel then
		return 0
	end
	
	-- Get the zone specific value needed for the calculations
	-- I assume levels 58-68 and 69-80 will be leveling in Outlands and Northrend, respectively.
	-- (I do this mainly to avoid missleading information when high-level players are in the old zones.)
	if charLevel >= 58 and charLevel <= 68 then
		zoneID = 3
	elseif charLevel >= 69 then -- no need for an upper limit
		zoneID = 4
	else
		zoneID = ZoneID()
	end
	
	local addValue = 45 -- Default, for the pre-tbc zones
	if zoneID == 3 then
		addValue = 235 -- Outlands
    elseif zoneID == 4 then 
        addValue = 580 -- Northrend
	end
	
	local XP = 0
	if(charLevel == mobLevel) then
		XP = (charLevel * 5) + addValue
	elseif (charLevel < mobLevel) then
		-- Find max mob level. All mobs higher than (charLevel + 4) are worth the same amount.
		local maxMobLevel = mobLevel
		if mobLevel > charLevel + 4 then 
			maxMobLevel = charLevel + 4 
		end
		XP = ((charLevel * 5 + addValue)) * (1 + (0.05 * (maxMobLevel - charLevel)))
	elseif (charLevel > mobLevel) then
		local ZD = ZeroDifference(charLevel)
		XP = ((charLevel * 5 + addValue)) * (1 - ((charLevel - mobLevel) / ZD))
	end
	
	return XP
end

-- Calculates the XP a hunter's pet gains when killing a mob.
function PetXP(playerLevel, petLevel, mobLevel)
	-- Set some default values.
	if playerLevel == nil then
		playerLevel = UnitLevel("player");
	end
	if petLevel == nil then
		petLevel = playerLevel - 1;
	end
	if mobLevel == nil then
		mobLevel = playerLevel;
	end
	
	local petXP = MobXP(playerLevel, (playerLevel + (mobLevel - petLevel)));
	return petXP;
end

---
-- Extracts the XP amount from a chat XP gain message, as well as the mob name if
-- the message was generated by a kill, rather then a quest gain.
---
function ParseChatXPMessage(message, locale)
    if locale == nil then
        locale = "enUS"
    end
    local pattern, mob, xp;
	
	--- Check if this is a quest or a mob
	if(string.find(message, ",") == nil) then
		pattern = LOCALE_XPGAIN_MSG[locale];
		xp = strmatch(message, pattern);
	else
		pattern = LOCALE_KILL_MSG[locale];
		mob, xp = strmatch(message, pattern);
	end
	
	return xp, mob;
end

function round(input, precision, roundDown)
	precision = 10^(precision or 2)
	local altered = input * precision
	if roundDown then
		return math.floor(altered) / precision
	else
		if altered - math.floor(altered) >= 0.5 then
			return math.ceil(altered) / precision
		else
			return math.floor(altered) / precision
		end
	end
end

-- Returns the input number with commas inserted.
-- TODO: Clean this mess up.
function NumberFormat(input) -- /run cDisplay:Print(NumberFormat(127))
	local strVersion = tostring(input)
	local strLength = strlen(strVersion)
	
	local numVersion = ""
	local fraction = nil
	
	local i = 0
	while i < strLength do
		i = i + 1
		local current = strsub(strVersion, i, i)
		if current == "." then
			fraction = strsub(strVersion, i + 1)
			break
		else
			numVersion = numVersion .. current
		end
	end
	
	local output = ""
	strLength = strlen(numVersion)
	local i = 0
	while i < strLength do
		if i > 0 and mod(i, 3) == 0 then
			output = "," .. output
		end
		output = strsub(numVersion, (strLength - i), (strLength - i)) .. output
		i = i + 1
	end
	if fraction then
		return output .. "." .. fraction
	else
		return output
	end
end

function ShrinkNumber(input)
	input = tonumber(input)
	if input < 100000 then
		return NumberFormat(input)
	else
		local units = {"", "K", "M", "B"}
		local index = 1
		local output = input
		while output > 1000 and index < # units do
			output = output / 1000
			index = index + 1
		end
		local precision = 2
		if output < 10 then
			precision = 2
		elseif output < 100 then
			precision = 1
		else
			precision = 0
		end
		return NumberFormat(tostring(round(output, precision, true))) .. units[index]
	end
end