----------------------
-- Event callbacks
----------------------
function MainOnload()
    -- Register Events
    this:RegisterEvent("CHAT_MSG_COMBAT_XP_GAIN");
    this:RegisterEvent("PLAYER_LOGIN");
    this:RegisterEvent("PLAYER_LEVEL_UP");
    this:RegisterEvent("PLAYER_XP_UPDATE");
	this:RegisterEvent("UNIT_PET");
	this:RegisterEvent("UNIT_PET_EXPERIENCE");
	this:RegisterEvent("PET_UI_UPDATE");
    
    -- Register slash commands
    SLASH_XTOLEVEL1 = "/xtolevel";
    SLASH_XTOLEVEL2 = "/xtl";
    SlashCmdList["XTOLEVEL"] = OnSlashCommand;
end

function MainOnPlayerLogin()
	VerifyConfig()

    cPlayer:Initialize(sData.player.killAverage, sData.player.questAverage)
	cPet:Initialize(sData.pet.killAverage)
    cDisplay.Average:Initialize()
    cDisplay.Config:Initialize()
end

function OnPlayerLevelUp(newLevel)
    cDisplay.Floating:PrintLevel(newLevel)
    cDisplay.Chat:PrintLevel(newLevel)
	
	cPlayer.level = newLevel
	cPlayer:ClearKillList(cPlayer:GetAverageKillXP())
	cPlayer:ClearQuestList(cPlayer:GetAverageQuestXP())
	
	cPet:Initialize()
	cPet:ClearKillList(cPet:GetAverageKillXP())
	
	cDisplay.Average:Update()
	UpdateDataObject()
end

function OnChatXPGain(message)
    -- Note that this event is fired by both kills and quest hand-ins.
    local xp, mobName = ParseChatXPMessage(message, LOCALE_CLIENT)
    xp = tonumber(xp)
    
    -- See if it is a kill or a quest
    if mobName ~= nil then
        local unrestedXP = cPlayer:AddKill(xp, mobName)
        
        if sConfig.messages.playerFloating or sConfig.messages.playerChat then
            local killsRequired = cPlayer:GetKillsRequired(unrestedXP)
            if killsRequired > 0 then
                cDisplay.Floating:PrintKill(mobName, killsRequired)
                cDisplay.Chat:PrintKill(mobName, killsRequired)
				cPet.nextMobName = mobName -- Sets the next mob to be displayed for the pet message.
            end
        end

    else
        cPlayer:AddQuest(xp)
        
        if sConfig.messages.playerFloating or sConfig.messages.playerChat then
            local questsRequired = cPlayer:GetQuestsRequired(xp)
            if questsRequired > 0 then
                cDisplay.Floating:PrintQuest(questsRequired)
                cDisplay.Chat:PrintQuest(questsRequired)
            end
        end
    end
end

function OnPlayerXPUpdate()
    cPlayer:SyncData()
    cDisplay.Average:Update()
	UpdateDataObject()
    
    sData.player.killAverage = cPlayer:GetAverageKillXP()
    sData.player.questAverage = cPlayer:GetAverageQuestXP()
end

function OnUnitPet(arg1)
	if arg1 == "player" then
		cPet:Initialize()
		cDisplay.Average:Update()
	end
end

function OnUnitPetExperience()
	local xpGained, killsRemaining, mobName;
	xpGained = cPet:Update()
	if xpGained > 0 then
		killsRemaining = ceil((cPet.maxXP - cPet.xp) / xpGained)
		if killsRemaining > 0 then
			mobName = cPet:GetMobName()
			cDisplay.Floating:PrintPetKill(cPet:GetName(), mobName, killsRemaining)
			cDisplay.Chat:PrintPetKill(cPet:GetName(), mobName, killsRemaining)
		end
	end
	cDisplay.Average:Update()
end

function OnPetUiUpdate()
	-- This event only fires after a new pet has been trained (as far as I know
	-- Included here to circumvent a problem with the UNIT_PET event firing to early when a new pet is trained.
	cPet:Initialize()
	cDisplay.Average:Update()
end

function OnSlashCommand(arg1)
	if true then
		cDisplay:Print(ShrinkNumber(tonumber(arg1)))
	else
		cDisplay.Config.waterfall:Open('ConfigWindow', 'general')
	end
end

----------------------
-- Main event handler
----------------------
function MainOnEvent()
	if event == "PLAYER_LOGIN" then
		MainOnPlayerLogin()
	elseif event == "PLAYER_LEVEL_UP" then
		OnPlayerLevelUp(arg1)
	elseif event == "CHAT_MSG_COMBAT_XP_GAIN" then
		OnChatXPGain(arg1)
    elseif event == "PLAYER_XP_UPDATE" then
        OnPlayerXPUpdate()
	elseif (event == "UNIT_PET") then
		OnUnitPet(arg1)
	elseif event == "UNIT_PET_EXPERIENCE" then
		OnUnitPetExperience();
	elseif event == "PET_UI_UPDATE" then
		OnPetUiUpdate()
	end
end
