﻿cDisplay = 
{  
    ---
    -- A shortcut to add a message to the default chat window
    printStyle = {
        white = { r=1.0, g=1.0, b=1.0, group=54, addToStart=false },
        gray = { r=0.5, g=0.5, b=0.5, group=53, addToStart=false }
    },
    Print = function(self, message, style)
        if style == nil then
            style = self.printStyle.white
        end
        DEFAULT_CHAT_FRAME:AddMessage(message, style.r, style.g, style.b, style.group, style.addToStart);
    end,

    ---
    -- Average Window controls
    ---
    Average = 
    {
        window = nil,
        lines = {},
        backdrop = nil,
    
        -- Window Style
        visible = true, -- Note that this should be treated as a private member. Use the SetVisibility function.
        showBackground = true,
        textMargin = 5,
        lineSpacing = 2,
        
        ---
        -- Creates the window
        Initialize = function(self)
            if self.window ~= nil then
                return
            end
        
            self.window = XToLevelDisplay
            self.backdrop = self.window:GetBackdrop();
            
            -- Create the lines
            self.lines.header = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_h1')
            self.lines.header:SetText('XToLevel')
            self.lines.header.group = "header"
            self.lines.header.tabIndex = 1
            
            self.lines.playerKills = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.playerKills:SetText(LOCALE_MESSAGES[LOCALE_DISPLAY]['Kills'] .. ': ...')
            self.lines.playerKills.group = "player"
            self.lines.playerKills.tabIndex = 2
            
            self.lines.playerQuests = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.playerQuests:SetText(LOCALE_MESSAGES[LOCALE_DISPLAY]['Quests'] .. ': ...')
            self.lines.playerQuests.group = "player"
            self.lines.playerQuests.tabIndex = 3
			
			self.lines.petKills = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.petKills:SetText(LOCALE_MESSAGES[LOCALE_DISPLAY]['Pet'] .. ': ...')
            self.lines.petKills.group = "pet"
            self.lines.petKills.tabIndex = 4
            
            self:Update()
            self:updateBackdrop()
        end,
        
        ---
        -- Updates the window
        Update = function(self)
            if self.window == nil then
                return
            end
            
            if not sConfig.averageDisplay.visible then
                self.window:Hide()
                return
            elseif not cPlayer.isActive and not cPet.isActive then
				self.window:Hide()
                return
			else
				self.window:Show()
            end
			
			if sConfig.averageDisplay.header then
				self.lines.header:Show()
			end
			
			if not cPlayer.isActive then
				self.lines.playerKills:Hide()
				self.lines.playerQuests:Hide()
			else
				self.lines.playerKills:Show()
				self.lines.playerQuests:Show()
			end
			if not cPet.isActive then
				self.lines.petKills:Hide()
			else
				self.lines.petKills:Show()
			end
			
            -- Show / Hide the lines and update the anchors
            -- The elements are re-arraged into a temp array according to their tabIndex
            local iLines = {}
            for eName, eElem in pairs(self.lines) do
                iLines[eElem.tabIndex] = { name=eName, elem=eElem }
            end
            
            local nextAnchor = self.window
            local nextMarginV = self.textMargin
            local nextMarginH = self.textMargin
            local nextPoint = "TOPLEFT"
            local currentIndent =  -nextMarginH
            for index, value in ipairs(iLines) do
                local name = value.name
                local elem = value.elem
                
                elem:ClearAllPoints()
                if sConfig.averageDisplay[name] and elem:IsShown() then
                    -- Clear text indent if the previous element was in the same group
                    if nextAnchor.group == elem.group then
                        nextMarginH = 0
                    else
                        currentIndent = currentIndent + nextMarginH
                    end
                    elem:SetPoint("TOPLEFT", nextAnchor, nextPoint, nextMarginH, -nextMarginV)
                    elem.lineIndent = currentIndent
                    
                    nextAnchor = elem
                    nextMarginV = self.lineSpacing
                    nextPoint = "BOTTOMLEFT"
                else
                    elem:Hide()
                end
            end
            
            -- Update messages for the visible elements
            if self.lines.header:IsShown() then
                self.lines.header:SetText('XToLevel')
            end
			
			if cPlayer.isActive then
				if self.lines.playerKills:IsShown() then
					self.lines.playerKills:SetText(LOCALE_MESSAGES[LOCALE_DISPLAY]['Kills'] .. ': ' .. cPlayer:GetAverageKillsRemaining())
				end
				if self.lines.playerQuests:IsShown() then
					self.lines.playerQuests:SetText(LOCALE_MESSAGES[LOCALE_DISPLAY]['Quests'] .. ': ' .. cPlayer:GetAverageQuestsRemaining())
				end
			end
			
			if cPet.isActive then
				if self.lines.petKills:IsShown() then
					self.lines.petKills:SetText(LOCALE_MESSAGES[LOCALE_DISPLAY]['Pet'] .. ': ' .. cPet:GetAverageKillsRemaining())
				end
			end
            
            -- Calculate the height and width of the lines
            local maxWidth = 0
            local totalHeight = self.textMargin * 2
            for name, value in pairs(self.lines) do
                if value:IsVisible() then
                    local currentWidth = value:GetWidth() + (value.lineIndent or 0)
                    if currentWidth  > maxWidth then
                        maxWidth = currentWidth
                    end
                    totalHeight = totalHeight + value:GetHeight() + self.lineSpacing
                end
            end
            local totalWidth = maxWidth + (self.textMargin * 2)
            
            -- Set the display box size
            self.window:SetWidth(totalWidth)
            self.window:SetHeight(totalHeight)
            
            self:updateBackdrop()
        end,
        
        ---
        -- Updates the backdrop
        updateBackdrop = function(self)
            if self.window == nil then
                return
            end
            
            if sConfig.averageDisplay.backdrop then
                self.window:SetBackdrop(self.backdrop);
                self.window:SetBackdropColor("0.0", "0.75", "0.5", "0.75");
                self.window:SetBackdropBorderColor("0.0", "0.0", "0.0", "1.0");
            else
                self.window:SetBackdrop(nil);
            end
        end,
        
        -- Show / Hide the window
        ToggleVisibility = function(self, newValue)
            if self.window == nil then
                return
            end
        
            if(newValue) then
                self.visible = newValue
            else
                self.visible = not self.visible
            end
            
            if self.visible then
                self.window:Show()
            else
                self.window:Hide()
            end
        end
    },
    
    ---
    -- Controls for the floating display
    -- The settings check is added here to simplify the calling code.
    ---
    Floating = 
    {
        killStyle =  { r=0.5, g=1.0, b=0.7, group=56, fade=5 },
		petStyle =  { r=1.0, g=0.7, b=0.5, group=57, fade=5 },
        questStyle = { r=0.5, g=1.0, b=0.7, group=56, fade=5 },
        levelStyle = { r=0.35, g=1.0, b=0.35, group=56, fade=6 },
        
        PrintKill = function(self, mobName, mobsRequired)
            if sConfig.messages.playerFloating then
                local message = mobsRequired .." ".. mobName .. LOCALE_MESSAGES[LOCALE_DISPLAY]['KillsNeeded']
                self:Print(message, self.killStyle)
            end
        end,
		
		PrintPetKill = function(self, petName, mobName, mobsRequired)
            if sConfig.messages.petFloating then -- sConfig.messages.petFloating
                local message = mobsRequired .. " " .. mobName .. LOCALE_MESSAGES[LOCALE_DISPLAY]['PetKillsNeeded'] .. petName
                self:Print(message, self.petStyle)
            end
        end,
        
        PrintQuest = function(self, questsRequired)
            if sConfig.messages.playerFloating then
                local message = questsRequired .. LOCALE_MESSAGES[LOCALE_DISPLAY]['QuestsNeeded']
                self:Print(message, self.questStyle)
            end
        end,
        
        PrintLevel = function(self, level)
            if sConfig.messages.playerFloating then
                local message = LOCALE_MESSAGES[LOCALE_DISPLAY]['LevelReached']
                self:Print(message, self.levelStyle)
            end
        end,
        
        Print = function(self, text, style)
            UIErrorsFrame:AddMessage(text, style.r, style.g, style.b, style.group, style.fade);
        end
    },
    
    ---
    -- Controls for the chat display
    ---
    Chat = 
    {
        killStyle =  { r=0.5, g=1.0, b=0.7, group=56, addToStart=false },
		petStyle =  { r=1.0, g=0.7, b=0.5, group=57, fade=5, addToStart=false },
        questStyle = { r=0.5, g=1.0, b=0.7, group=56, addToStart=false },
        levelStyle = { r=0.35, g=1.0, b=0.35, group=56, addToStart=false},
        
        PrintKill = function(self, mobName, mobsRequired)
            if sConfig.messages.playerChat then
                local message = mobsRequired .." ".. mobName .. LOCALE_MESSAGES[LOCALE_DISPLAY]['KillsNeeded']
                cDisplay:Print(message, self.killStyle)
            end
        end,
		
		PrintPetKill = function(self, petName, mobName, mobsRequired)
            if sConfig.messages.petChat then -- sConfig.messages.petFloating
                local message = mobsRequired .. " " .. mobName .. LOCALE_MESSAGES[LOCALE_DISPLAY]['PetKillsNeeded'] .. petName
                cDisplay:Print(message, self.petStyle)
            end
        end,
        
        PrintQuest = function(self, questsRequired)
            if sConfig.messages.playerChat then
                local message = questsRequired .. LOCALE_MESSAGES[LOCALE_DISPLAY]['QuestsNeeded']
                cDisplay:Print(message, self.questStyle)
            end
        end,
        
        PrintLevel = function(self, level)
            if sConfig.messages.playerChat then
                local message = LOCALE_MESSAGES[LOCALE_DISPLAY]['LevelReached']
                cDisplay:Print(message, self.levelStyle)
            end
        end
    },
    
    ---
    -- Controls for the configuration window
    ---
    Config =
    {
        waterfall = nil,
        
        Initialize = function(self)
            self.waterfall = AceLibrary("Waterfall-1.0")
            self.waterfall:Register('ConfigWindow',
                'title',LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Title"],
                'colorR', 0.0,
                'colorG', 0.5,
                'colroB', 0.25,
                'tree', 
                {
                    {
                        text = LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["General"]["Name"],
                        id = 'general',
                        isOpen = true,
                        isSelected = true
                    },
                    {
                        text = LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["Messages"]["Name"],
                        id = 'messages',
                        isOpen = true,
                        isSelected = true
                    },
                    {
                        text = LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["Name"],
                        id = 'average',
                        isOpen = true,
                        isSelected = true
                    },
					{
                        text = LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["Name"],
                        id = 'ldb',
                        isOpen = true,
                        isSelected = true
                    }
                },
             'children', 
                function(id)
                    if id=="general" then
                        if LOCALE_CLIENT ~= "enUS" then
                            self.waterfall:AddControl(
                                'type', 'checkbox',
                                'text', LOCALE_CONFIG_TEXT[LOCALE_CLIENT]["Categories"]["General"]["UseEnglish"],
                                'checked', sConfig["general"]["useEnglish"],
                                'fullRefresh', true,
                                'setFunc', function()
                                    sConfig.general.useEnglish = not sConfig.general.useEnglish
                                    if sConfig.general.useEnglish then
                                        LOCALE_DISPLAY = "enUS"
                                    else
                                        LOCALE_DISPLAY = LOCALE_CLIENT
                                    end
                                    cDisplay.Average:Update()
                                    self.waterfall:Close('ConfigWindow');
                                    self:Initialize()
                                    self.waterfall:Open('ConfigWindow', 'general');
                                end
                            )
                        else
                            self.waterfall:AddControl(
                            'type', 'label',
                            'text', "No options available"
                        )
                        end
                        
                        return LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["General"]["Name"]
                    elseif id=='messages' then
                        self.waterfall:AddControl(
                            'type', 'label',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["Messages"]["PlayerMessagesLabel"]
                        )
                        self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["Messages"]["ShowInChat"],
                            'checked', sConfig['messages']['playerChat'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['messages']['playerChat'] = not sConfig['messages']['playerChat'] 
                                cDisplay.Average:Update()
                            end
                        )
                        self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["Messages"]["ShowFloating"],
                            'checked', sConfig['messages']['playerFloating'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['messages']['playerFloating'] = not sConfig['messages']['playerFloating'] 
                                cDisplay.Average:Update()
                            end
                        )
						
						self.waterfall:AddControl(
                            'type', 'label',
                            'text', ""
                        )
						
						self.waterfall:AddControl(
                            'type', 'label',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["Messages"]["PetMessagesLabel"]
                        )
                        self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["Messages"]["ShowInChat"],
                            'checked', sConfig['messages']['petChat'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['messages']['petChat'] = not sConfig['messages']['petChat'] 
                                cDisplay.Average:Update()
                            end
                        )
                        self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["Messages"]["ShowFloating"],
                            'checked', sConfig['messages']['petFloating'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['messages']['petFloating'] = not sConfig['messages']['petFloating'] 
                                cDisplay.Average:Update()
                            end
                        )
                        
                        return LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["Messages"]["Name"]
                    elseif id=='average' then                        
                        self.waterfall:AddControl(
                            'type', 'label',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["AppearenceLabel"]
                        )
                        self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["ShowAverageDisplay"],
                            'checked', sConfig['averageDisplay']['visible'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['averageDisplay']['visible'] = not sConfig['averageDisplay']['visible']
                                cDisplay.Average:Update()
                            end
                        )
                        self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["ShowHeader"],
                            'checked', sConfig['averageDisplay']['header'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['averageDisplay']['header'] = not sConfig['averageDisplay']['header']
                                cDisplay.Average:Update()
                            end
                        )
                        self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["ShowFrame"],
                            'checked', sConfig['averageDisplay']['backdrop'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['averageDisplay']['backdrop'] = not sConfig['averageDisplay']['backdrop']
                                cDisplay.Average:Update()
                            end
                        )
                        
                        self.waterfall:AddControl(
                            'type', 'label',
                            'text', ""
                        )
                        
                        self.waterfall:AddControl(
                            'type', 'label',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["DataLabel"]
                        )
                        self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["ShowKills"],
                            'checked', sConfig['averageDisplay']['playerKills'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['averageDisplay']['playerKills'] = not sConfig['averageDisplay']['playerKills']
                                cDisplay.Average:Update()
                            end
                        )
                        self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["ShowQuests"],
                            'checked', sConfig['averageDisplay']['playerQuests'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['averageDisplay']['playerQuests'] = not sConfig['averageDisplay']['playerQuests']
                                cDisplay.Average:Update()
                            end
                        )
						self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["ShowPet"],
                            'checked', sConfig['averageDisplay']['petKills'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['averageDisplay']['petKills'] = not sConfig['averageDisplay']['petKills']
                                cDisplay.Average:Update()
                            end
                        )
                        
                        self.waterfall:AddControl(
                            'type', 'label',
                            'text', ""
                        )
                        
                        self.waterfall:AddControl(
                            'type', 'label',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["BehaviorLabel"]
                        )
                        self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["LockAverageDisplay"],
                            'checked', not sConfig['general']['allowDrag'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['general']['allowDrag'] = not sConfig['general']['allowDrag']
                                cDisplay.Average:Update()
                            end
                        )
                        self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["AllowSettingsClick"],
                            'checked', sConfig['general']['allowSettingsClick'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['general']['allowSettingsClick'] = not sConfig['general']['allowSettingsClick']
                            end
                        )
                        
                        return LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["AverageDisplay"]["Name"]
					elseif id=='ldb' then
						self.waterfall:AddControl(
                            'type', 'label',
                            'text', "|cFFaaaaaaThis controls how XToLevel is represented in|r"
                        )
						self.waterfall:AddControl(
                            'type', 'label',
                            'text', "|cFFaaaaaaLDB front-end addons, such as TitanPanel.|r"
                        )
						self.waterfall:AddControl(
                            'type', 'label',
                            'text', ""
                        )
						
						self.waterfall:AddControl(
                            'type', 'label',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["GeneralLabel"]
                        )
						self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["ShowText"],
                            'checked', sConfig['ldb']['showText'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['ldb']['showText'] = not sConfig['ldb']['showText']
                                UpdateDataObject()
                            end
                        )
						self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["ShowIcon"],
                            'checked', sConfig['ldb']['showIcon'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['ldb']['showIcon'] = not sConfig['ldb']['showIcon']
                                UpdateDataObject()
                            end
                        )
						self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["ShowLabel"],
                            'checked', sConfig['ldb']['showLabel'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['ldb']['showLabel'] = not sConfig['ldb']['showLabel']
                                UpdateDataObject()
                            end
                        )
						self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["UseCustomColors"],
                            'checked', sConfig['ldb']['customColors'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['ldb']['customColors'] = not sConfig['ldb']['customColors']
                                UpdateDataObject()
                            end
                        )
						
						self.waterfall:AddControl(
                            'type', 'label',
                            'text', ""
                        )
					
                        self.waterfall:AddControl(
                            'type', 'label',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["TextLabel"]
                        )
                        self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["ShowTextKills"],
                            'checked', sConfig['ldb']['text']['playerKills'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['ldb']['text']['playerKills'] = not sConfig['ldb']['text']['playerKills']
                                UpdateDataObject()
                            end
                        )
						self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["ShowTextQuests"],
                            'checked', sConfig['ldb']['text']['playerQuests'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['ldb']['text']['playerQuests'] = not sConfig['ldb']['text']['playerQuests']
                                UpdateDataObject()
                            end
                        )
						self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["ShowTextPet"],
                            'checked', sConfig['ldb']['text']['petKills'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['ldb']['text']['petKills'] = not sConfig['ldb']['text']['petKills']
                                UpdateDataObject()
                            end
                        )
						self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["ShowTextXP"],
                            'checked', sConfig['ldb']['text']['xpPogress'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['ldb']['text']['xpPogress'] = not sConfig['ldb']['text']['xpPogress']
                                UpdateDataObject()
                            end
                        )
						self.waterfall:AddControl(
                            'type', 'label',
                            'text', ""
                        )
						self.waterfall:AddControl(
                            'type', 'label',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["TooltipLabel"]
                        )
						self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["ShowTooltipDetails"],
                            'checked', sConfig['ldb']['tooltip']['showDetails'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['ldb']['tooltip']['showDetails'] = not sConfig['ldb']['tooltip']['showDetails']
                                UpdateDataObject()
                            end
                        )
						self.waterfall:AddControl(
                            'type', 'checkbox',
                            'text', LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["ShowTooltipExperience"],
                            'checked', sConfig['ldb']['tooltip']['showExperience'],
                            'fullRefresh', true,
                            'setFunc', function() 
                                sConfig['ldb']['tooltip']['showExperience'] = not sConfig['ldb']['tooltip']['showExperience']
                                UpdateDataObject()
                            end
                        )
                        
                        return LOCALE_CONFIG_TEXT[LOCALE_DISPLAY]["Categories"]["LDB"]["Name"]
                    end
                end 
            )
        end
    }
}